using FitConnect.Models;
using FluentAssertions;
using Microsoft.Extensions.Logging;

namespace E2ETest;

[TestFixture]
public class StraightForwardTest : EndToEndTestBase {
    private string _caseId = null!;
    private string _submissionId = null!;

    [Order(10)]
    [Test]
    public void SendingSubmission() {
        var submission = Sender.WithDestination(Settings.DestinationId)
            .WithServiceType("Straight forward test", Settings.LeikaKey)
            .WithAttachments(new Attachment("Test.pdf", "A simple PDF"))
            .WithAttachments("Test_2.pdf", "Clean Code",
                new FileStream("Test_2.pdf", FileMode.Open))
            .WithJsonData(@"{""data"":""value""}")
            .Submit();

        _caseId = submission.CaseId!;
        _submissionId = submission.Id!;

        _caseId.Should().NotBeNull();
        _submissionId.Should().NotBeNull();
    }

    [Test]
    [Order(20)]
    public void Sender_GetSubmissionState() {
        // Act
        var status = Sender.GetStatusForSubmission(_caseId, Settings.DestinationId);

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }

    [Test]
    [Order(30)]
    public void Subscriber_GetSubmissionState() {
        // Act
        var status = Subscriber.GetStatusForSubmission(_caseId, Settings.DestinationId);

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }

    [Test]
    [Order(40)]
    public void RequestSubmission() {
        var subscriberWithSubmission = Subscriber.RequestSubmission(_submissionId);

        var data = subscriberWithSubmission.GetDataJson();
        Logger.LogInformation("Data {Data}", data);

        data.Should().Be(@"{""data"":""value""}");

        var attachments = subscriberWithSubmission.GetAttachments();
        attachments.Should().Contain(a => a.Filename == "Test.pdf");
        attachments.Should().Contain(a => a.Filename == "Test_2.pdf");
        attachments.Count().Should().Be(2);

        subscriberWithSubmission.AcceptSubmission();
    }

    [Test]
    [Order(50)]
    public void Sender_GetSubmissionState_AfterAccepting() {
        // Act
        var status = Sender.GetStatusForSubmission(_caseId, Settings.DestinationId);

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }
}
